@@ -177,8 +177,7 @@ gb_widget_write_source (GtkWidget * widget,
 	{
 	  /* For empty clist/ctree titles, we increment the 'last_child'
 	     written value. */
-	  if (child_name && (!strcmp (child_name, "CList:title")
-			     || !strcmp (child_name, "CTree:title")))
+	  if (child_name && !strcmp (child_name, GladeChildCListTitle))
 	    {
 	      gint col = GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (parent),
 							       "last_child"));
