@@ -25,6 +25,8 @@
 #include <gtk/gtklabel.h>
 #include <gtk/gtklist.h>
 #include <gtk/gtklistitem.h>
+#include <gtk/gtkmenuitem.h>
+#include <gtk/gtknotebook.h>
 #include <gtk/gtkeventbox.h>
 #include "../gb.h"
 
@@ -38,18 +40,17 @@
 static GbWidget gbwidget;
 
 static gchar *Label = "GtkLabel::label";
+static gchar *UseUnderline = "GtkLabel::use_underline";
+static gchar *UseMarkup = "GtkLabel::use_markup";
 static gchar *Justify = "GtkLabel::justify";
 static gchar *Wrap = "GtkLabel::wrap";
 static gchar *XAlign = "Label|GtkMisc::xalign";
 static gchar *YAlign = "Label|GtkMisc::yalign";
 static gchar *XPad = "Label|GtkMisc::xpad";
 static gchar *YPad = "Label|GtkMisc::ypad";
+static gchar *Selectable = "GtkLabel::selectable";
 
-static gchar *AccelTarget = "GtkLabel::focus_target";
-
-/* This is only saved to the XML, and only when AccelTarget is set to Auto.
-   It is not loaded or used in Glade. */
-static gchar *DefaultAccelTarget = "GtkLabel::default_focus_target";
+static gchar *FocusTarget = "GtkLabel::mnemonic_widget";
 
 
 static const gchar *GbJustifyChoices[] =
@@ -113,70 +114,142 @@ gb_label_new (GbWidgetNewData * data)
 
 
 
+void
+gb_label_create_standard_properties (GtkWidget * widget,
+				     GbWidgetCreateArgData * data,
+				     const char *label_p,
+				     const char *use_underline_p,
+				     const char *use_markup_p,
+				     const char *justify_p,
+				     const char *wrap_p,
+				     const char *selectable_p,
+				     const char *xalign_p,
+				     const char *yalign_p,
+				     const char *xpad_p,
+				     const char *ypad_p,
+				     const char *focus_target_p)
+{
+  GtkWidget *combo;
+
+  property_add_text (label_p, _("Label:"), _("The text to display"), 2);
+  property_add_bool (use_underline_p, _("Use Underline:"),
+		     _("If the text includes an underlined accelerator character"));
+  property_add_bool (use_markup_p, _("Use Markup:"),
+		     _("If the text includes pango markup"));
+  property_add_choice (justify_p, _("Justify:"),
+		       _("The justification of the lines of the label"),
+		       GbJustifyChoices);
+  property_add_bool (wrap_p, _("Wrap Text:"),
+		     _("If the text is wrapped to fit within the width of the label"));
+  property_add_bool (selectable_p, _("Selectable:"),
+		     _("If the label text can be selected with the mouse"));
+  property_add_float_range (xalign_p, _("X Align:"),
+			    _("The horizontal alignment of the entire label"),
+			    0, 1, 0.01, 0.1, 0.01, 2);
+  property_add_float_range (yalign_p, _("Y Align:"),
+			    _("The vertical alignment of the entire label"),
+			    0, 1, 0.01, 0.1, 0.01, 2);
+  property_add_int_range (xpad_p, _("X Pad:"), _("The horizontal padding"),
+			  0, 1000, 1, 10, 1);
+  property_add_int_range (ypad_p, _("Y Pad:"), _("The vertical padding"),
+			  0, 1000, 1, 10, 1);
+  property_add_combo (focus_target_p, _("Focus Target:"),
+		      _("The widget to set the keyboard focus to when the underlined accelerator key is used"),
+		      NULL);
+  combo = property_get_value_widget (focus_target_p);
+  gtk_entry_set_editable (GTK_ENTRY (GTK_COMBO (combo)->entry), FALSE);
+  gtk_combo_set_value_in_list (GTK_COMBO (combo), TRUE, TRUE);
+}
+
+
 /*
  * Creates the components needed to edit the extra properties of this widget.
  */
 static void
 gb_label_create_properties (GtkWidget * widget, GbWidgetCreateArgData * data)
 {
-  GtkWidget *combo;
-
-  property_add_text (Label, _("Label:"), _("The text to display"), 2);
-  property_add_choice (Justify, _("Justify:"),
-		       _("The justification of the lines of the label"),
-		       GbJustifyChoices);
-  property_add_bool (Wrap, _("Wrap Text:"),
-		     _("If the text is wrapped to fit within the width of the label"));
-  property_add_float_range (XAlign, _("X Align:"),
-			    _("The horizontal alignment of the entire label"),
-			    0, 1, 0.01, 0.1, 0.01, 2);
-  property_add_float_range (YAlign, _("Y Align:"),
-			    _("The vertical alignment of the entire label"),
-			    0, 1, 0.01, 0.1, 0.01, 2);
-  property_add_int_range (XPad, _("X Pad:"), _("The horizontal padding"),
-			  0, 1000, 1, 10, 1);
-  property_add_int_range (YPad, _("Y Pad:"), _("The vertical padding"),
-			  0, 1000, 1, 10, 1);
-  property_add_combo (AccelTarget, _("Focus Target:"),
-		      _("The widget to set the keyboard focus to when the underlined accelerator key is used"),
-		      NULL);
-  combo = property_get_value_widget (AccelTarget);
-  gtk_entry_set_editable (GTK_ENTRY (GTK_COMBO (combo)->entry), FALSE);
-  gtk_combo_set_value_in_list (GTK_COMBO (combo), TRUE, TRUE);
+  gb_label_create_standard_properties (widget, data,
+				       Label, UseUnderline, UseMarkup,
+				       Justify, Wrap, Selectable,
+				       XAlign, YAlign, XPad, YPad,
+				       FocusTarget);
 }
 
 
 
-/*
- * Gets the properties of the widget. This is used for both displaying the
- * properties in the property editor, and also for saving the properties.
- */
-static void
-gb_label_get_properties (GtkWidget * widget, GbWidgetGetArgData * data)
+/* This tries to find a parent widget of a label that would be used for
+   mnemonic activation, e.g. a button or a menuitem. If it finds one, it
+   returns TRUE. */
+static gboolean
+gb_label_find_mnemonic_widget (GtkWidget *widget)
 {
-  gchar *label_text;
+  GtkWidget *parent;
+
+  parent = widget->parent;
+
+  while (parent)
+    {
+      if (GTK_WIDGET_GET_CLASS (parent)->activate_signal)
+	{
+	  return TRUE;
+	}
+      if (GTK_IS_MENU_ITEM (parent))
+	{
+	  return TRUE;
+	}
+
+      parent = parent->parent;
+    }
+
+  return FALSE;
+}
+
+
+void
+gb_label_get_standard_properties (GtkWidget * widget,
+				  GbWidgetGetArgData * data,
+				  const char *label_p,
+				  const char *use_underline_p,
+				  const char *use_markup_p,
+				  const char *justify_p,
+				  const char *wrap_p,
+				  const char *selectable_p,
+				  const char *xalign_p,
+				  const char *yalign_p,
+				  const char *xpad_p,
+				  const char *ypad_p,
+				  const char *focus_target_p)
+{
+  const gchar *label_text;
   gint i;
 
-  label_text = glade_util_get_label_text (widget);
-  gb_widget_output_translatable_text (data, Label, label_text);
+  label_text = gtk_label_get_label (GTK_LABEL (widget));
+  gb_widget_output_translatable_text (data, label_p, label_text);
+
+  gb_widget_output_bool (data, use_underline_p,
+			 gtk_label_get_use_underline (GTK_LABEL (widget)));
+  gb_widget_output_bool (data, use_markup_p,
+			 GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (widget), use_markup_p)));
 
   for (i = 0; i < sizeof (GbJustifyValues) / sizeof (GbJustifyValues[0]); i++)
     {
       if (GbJustifyValues[i] == GTK_LABEL (widget)->jtype)
-	gb_widget_output_choice (data, Justify, i, GbJustifySymbols[i]);
+	gb_widget_output_choice (data, justify_p, i, GbJustifySymbols[i]);
     }
-  gb_widget_output_bool (data, Wrap, GTK_LABEL (widget)->wrap);
-  gb_widget_output_float (data, XAlign, GTK_MISC (widget)->xalign);
-  gb_widget_output_float (data, YAlign, GTK_MISC (widget)->yalign);
-  gb_widget_output_int (data, XPad, GTK_MISC (widget)->xpad);
-  gb_widget_output_int (data, YPad, GTK_MISC (widget)->ypad);
+  gb_widget_output_bool (data, wrap_p, GTK_LABEL (widget)->wrap);
+  gb_widget_output_bool (data, selectable_p,
+			 gtk_label_get_selectable (GTK_LABEL (widget)));
+  gb_widget_output_float (data, xalign_p, GTK_MISC (widget)->xalign);
+  gb_widget_output_float (data, yalign_p, GTK_MISC (widget)->yalign);
+  gb_widget_output_int (data, xpad_p, GTK_MISC (widget)->xpad);
+  gb_widget_output_int (data, ypad_p, GTK_MISC (widget)->ypad);
 
   /* Labels not in buttons may have a focus target widget. */
-  if (!glade_util_find_parent_button (widget))
+  if (!gb_label_find_mnemonic_widget (widget))
     {
       gchar *accel_target;
 
-      accel_target = gtk_object_get_data (GTK_OBJECT (widget), AccelTarget);
+      accel_target = gtk_object_get_data (GTK_OBJECT (widget), focus_target_p);
 
       /* If we're showing we need to display the list of possible focus target
 	 widgets. We walk the tree of widgets in this component, and if a
@@ -186,14 +259,14 @@ gb_label_get_properties (GtkWidget * widget, GbWidgetGetArgData * data)
 	  GList *focus_targets = NULL, *standard_items = NULL;
 	  GtkWidget *item, *combo;
 
-	  property_set_visible (AccelTarget, TRUE);
+	  property_set_visible (focus_target_p, TRUE);
 
 	  gb_label_get_focus_targets (gtk_widget_get_toplevel (widget),
 				      &focus_targets);
-	  property_set_combo_strings (AccelTarget, focus_targets);
+	  property_set_combo_strings (focus_target_p, focus_targets);
 	  g_list_free (focus_targets);
 
-	  combo = property_get_value_widget (AccelTarget);
+	  combo = property_get_value_widget (focus_target_p);
 
 	  item = gtk_list_item_new_with_label (_("Auto"));
 	  gtk_widget_show (item);
@@ -212,7 +285,7 @@ gb_label_get_properties (GtkWidget * widget, GbWidgetGetArgData * data)
 	    {
 	      accel_target = _("Auto");
 	    }
-	  gb_widget_output_combo (data, AccelTarget, accel_target);
+	  gb_widget_output_combo (data, focus_target_p, accel_target);
 	}
       else
 	{
@@ -224,22 +297,21 @@ gb_label_get_properties (GtkWidget * widget, GbWidgetGetArgData * data)
 	      if (glade_util_find_widget (gtk_widget_get_toplevel (widget),
 					  accel_target))
 		{
-		  gb_widget_output_combo (data, AccelTarget, accel_target);
+		  gb_widget_output_combo (data, focus_target_p, accel_target);
 		}
 	    }
 	  else
 	    {
 	      /* If no target has been set, and the label has an underlined
 		 key, we try to find a default target and save that. */
-	      if (strchr (label_text, '_'))
+	      if (gtk_label_get_use_underline (GTK_LABEL (widget)))
 		{
 		  GtkWidget *accel_target;
-		  gchar *signal_name;
 
-		  accel_target = glade_util_find_default_accelerator_target (widget, &signal_name);
+		  accel_target = glade_util_find_default_accelerator_target (widget);
 		  if (accel_target)
 		    {
-		      gb_widget_output_string (data, DefaultAccelTarget, gtk_widget_get_name (accel_target));
+		      gb_widget_output_string (data, focus_target_p, gtk_widget_get_name (accel_target));
 		    }
 		}
 	    }
@@ -249,11 +321,24 @@ gb_label_get_properties (GtkWidget * widget, GbWidgetGetArgData * data)
     {
       if (data->action == GB_SHOWING)
 	{
-	  property_set_visible (AccelTarget, FALSE);
+	  property_set_visible (focus_target_p, FALSE);
 	}
     }
+}
 
-  g_free (label_text);
+
+/*
+ * Gets the properties of the widget. This is used for both displaying the
+ * properties in the property editor, and also for saving the properties.
+ */
+static void
+gb_label_get_properties (GtkWidget * widget, GbWidgetGetArgData * data)
+{
+  gb_label_get_standard_properties (widget, data,
+				    Label, UseUnderline, UseMarkup,
+				    Justify, Wrap, Selectable,
+				    XAlign, YAlign, XPad, YPad,
+				    FocusTarget);
 }
 
 
@@ -264,7 +349,7 @@ gb_label_get_focus_targets (GtkWidget * widget,
   if (GTK_WIDGET_CAN_FOCUS (widget) && GB_IS_GB_WIDGET (widget))
     {
       *focus_targets = g_list_insert_sorted (*focus_targets,
-					     gtk_widget_get_name (widget),
+					     (gchar*) gtk_widget_get_name (widget),
 					     g_str_equal);
     }
 
@@ -281,23 +366,105 @@ gb_label_get_focus_targets (GtkWidget * widget,
  * Sets the properties of the widget. This is used for both applying the
  * properties changed in the property editor, and also for loading.
  */
-static void
-gb_label_set_properties (GtkWidget * widget, GbWidgetSetArgData * data)
+void
+gb_label_set_standard_properties (GtkWidget * widget,
+				  GbWidgetSetArgData * data,
+				  const char *label_p,
+				  const char *use_underline_p,
+				  const char *use_markup_p,
+				  const char *justify_p,
+				  const char *wrap_p,
+				  const char *selectable_p,
+				  const char *xalign_p,
+				  const char *yalign_p,
+				  const char *xpad_p,
+				  const char *ypad_p,
+				  const char *focus_target_p)
 {
   gchar *label, *justify, *accel_target;
+  const gchar *label_text;
   gfloat xalign, yalign;
   gint xpad, ypad, i;
-  gboolean set_label = FALSE, free_label = FALSE, wrap;
-  gboolean set_alignment = FALSE, set_padding = FALSE;
+  gboolean wrap, selectable, set_alignment = FALSE, set_padding = FALSE;
+  gboolean use_underline, use_markup;
+  gboolean set_label = FALSE;
 
-  label = gb_widget_input_text (data, Label);
-  /* We use parse_uline so letters can be underlined. */
+  use_underline = gb_widget_input_bool (data, use_underline_p);
   if (data->apply)
-    set_label = TRUE;
-  if (data->action == GB_APPLYING)
-    free_label = TRUE;
+    gtk_label_set_use_underline (GTK_LABEL (widget), use_underline);
 
-  justify = gb_widget_input_choice (data, Justify);
+  use_markup = gb_widget_input_bool (data, use_markup_p);
+  if (data->apply)
+    {
+      set_label = TRUE;
+      gtk_object_set_data (GTK_OBJECT (widget), use_markup_p,
+			   GINT_TO_POINTER (use_markup));
+    }
+
+  label = gb_widget_input_text (data, label_p);
+  if (data->apply)
+    {
+      set_label = TRUE;
+      label_text = label;
+    }
+  else
+    {
+      label_text = gtk_label_get_label (GTK_LABEL (widget));
+    }
+
+  if (set_label)
+    {
+      gboolean prev_use_markup;
+
+      use_markup = GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (widget),
+							 use_markup_p));
+
+      /* We try to parse the markup here, and if it isn't valid, we will
+	 turn use_markup off and show it as plain text. */
+      if (use_markup)
+	{
+	  GError *error = NULL;
+	  gunichar accel_marker = 0;
+	  PangoAttrList *attrs = NULL;
+	  gunichar accel_char = 0;
+	  gchar *text = NULL;
+
+	  if (gtk_label_get_use_underline (GTK_LABEL (widget)))
+	    accel_marker = '_';
+
+	  /* We check it is valid markup. If it isn't we will set "use_markup"
+	     to FALSE. Note that we don't need attrs, text, or accel_char but
+	     it seems to crash otherwise. */
+	  if (!pango_parse_markup (label_text, -1, accel_marker, &attrs, &text,
+				   &accel_char, &error))
+	    {
+	      use_markup = FALSE;
+	      g_error_free (error);
+	    }
+	  else
+	    {
+	      if (attrs)
+		pango_attr_list_unref (attrs);
+	      g_free (text);
+	    }
+	}
+
+      /* If we are turning use_markup off, we want to do that before setting
+	 the text. If we are turning it on, we want to do it after. */
+      prev_use_markup = gtk_label_get_use_markup (GTK_LABEL (widget));
+      if (!use_markup && prev_use_markup)
+	gtk_label_set_use_markup (GTK_LABEL (widget), use_markup);
+
+      gtk_label_set_label (GTK_LABEL (widget), label_text);
+
+      if (use_markup && !prev_use_markup)
+	gtk_label_set_use_markup (GTK_LABEL (widget), use_markup);
+    }
+
+  if (data->action == GB_APPLYING)
+    g_free (label);
+
+  justify = gb_widget_input_choice (data, justify_p);
   if (data->apply)
     {
       for (i = 0; i < sizeof (GbJustifyValues) / sizeof (GbJustifyValues[0]);
@@ -307,41 +474,26 @@ gb_label_set_properties (GtkWidget * widget, GbWidgetSetArgData * data)
 	      || !strcmp (justify, GbJustifySymbols[i]))
 	    {
 	      gtk_label_set_justify (GTK_LABEL (widget), GbJustifyValues[i]);
-	      set_label = TRUE;
 	      break;
 	    }
 	}
     }
 
-  wrap = gb_widget_input_bool (data, Wrap);
+  wrap = gb_widget_input_bool (data, wrap_p);
   if (data->apply)
-    {
-      gtk_label_set_line_wrap (GTK_LABEL (widget), wrap);
-      set_label = TRUE;
-    }
+    gtk_label_set_line_wrap (GTK_LABEL (widget), wrap);
 
-  /* GTK+ 1.2 bug workaround. If we change the justify property, we need to
-     reset the label or it isn't displayed correctly. */
-  if (set_label)
-    {
-      if (!label)
-	{
-	  label = glade_util_get_label_text (widget);
-	  free_label = TRUE;
-	}
+  selectable = gb_widget_input_bool (data, selectable_p);
+  if (data->apply)
+    gtk_label_set_selectable (GTK_LABEL (widget), selectable);
 
-      gtk_label_parse_uline (GTK_LABEL (widget), label);
-    }
-  if (free_label)
-    g_free (label);
-
-  xalign = gb_widget_input_float (data, XAlign);
+  xalign = gb_widget_input_float (data, xalign_p);
   if (data->apply)
     set_alignment = TRUE;
   else
     xalign = GTK_MISC (widget)->xalign;
 
-  yalign = gb_widget_input_float (data, YAlign);
+  yalign = gb_widget_input_float (data, yalign_p);
   if (data->apply)
     set_alignment = TRUE;
   else
@@ -350,13 +502,13 @@ gb_label_set_properties (GtkWidget * widget, GbWidgetSetArgData * data)
   if (set_alignment)
     gtk_misc_set_alignment (GTK_MISC (widget), xalign, yalign);
 
-  xpad = gb_widget_input_int (data, XPad);
+  xpad = gb_widget_input_int (data, xpad_p);
   if (data->apply)
     set_padding = TRUE;
   else
     xpad = GTK_MISC (widget)->xpad;
 
-  ypad = gb_widget_input_int (data, YPad);
+  ypad = gb_widget_input_int (data, ypad_p);
   if (data->apply)
     set_padding = TRUE;
   else
@@ -366,15 +518,15 @@ gb_label_set_properties (GtkWidget * widget, GbWidgetSetArgData * data)
     gtk_misc_set_padding (GTK_MISC (widget), xpad, ypad);
 
   /* Labels not in buttons may have a focus target widget. */
-  accel_target = gb_widget_input_combo (data, AccelTarget);
+  accel_target = gb_widget_input_combo (data, focus_target_p);
   if (data->apply)
     {
-      if (!glade_util_find_parent_button (widget))
+      if (!gb_label_find_mnemonic_widget (widget))
 	{
 	  if (!strcmp (accel_target, _("Auto")))
 	    accel_target = NULL;
 
-	  gtk_object_set_data_full (GTK_OBJECT (widget), AccelTarget,
+	  gtk_object_set_data_full (GTK_OBJECT (widget), focus_target_p,
 				    g_strdup (accel_target),
 				    accel_target ? g_free : NULL);
 	}
@@ -382,6 +534,20 @@ gb_label_set_properties (GtkWidget * widget, GbWidgetSetArgData * data)
 }
 
 
+/*
+ * Sets the properties of the widget. This is used for both applying the
+ * properties changed in the property editor, and also for loading.
+ */
+static void
+gb_label_set_properties (GtkWidget * widget, GbWidgetSetArgData * data)
+{
+  gb_label_set_standard_properties (widget, data,
+				    Label, UseUnderline, UseMarkup,
+				    Justify, Wrap, Selectable,
+				    XAlign, YAlign, XPad, YPad,
+				    FocusTarget);
+}
+
 
 /*
  * Adds menu items to a context menu which is just about to appear!
@@ -397,79 +563,54 @@ gb_label_set_properties (GtkWidget * widget, GbWidgetSetArgData * data)
  */
 
 
-
-/*
- * Writes the source code needed to create this widget.
- * You have to output everything necessary to create the widget here, though
- * there are some convenience functions to help.
- */
-static void
-gb_label_write_source (GtkWidget * widget, GbWidgetWriteSourceData * data)
+void
+gb_label_write_standard_source (GtkWidget * widget,
+				GbWidgetWriteSourceData * data,
+				const char *label_p,
+				const char *use_underline_p,
+				const char *use_markup_p,
+				const char *justify_p,
+				const char *wrap_p,
+				const char *selectable_p,
+				const char *xalign_p,
+				const char *yalign_p,
+				const char *xpad_p,
+				const char *ypad_p,
+				const char *focus_target_p)
 {
-  gchar *label_text, *signal_name, *target_name;
   GtkWidget *accel_target = NULL;
   gint i;
 
-  if (data->create_widget)
+  if (gtk_object_get_data (GTK_OBJECT (widget), use_markup_p))
+    source_add (data, "  gtk_label_set_use_markup (%s, TRUE);\n",
+		data->wname);
+
+  /* If there is an underlined accelerator, set up the accel signal.
+     If the label is in a button or something similar, this is now setup
+     automatically by GTK+. If the accelerator is being used to set focus
+     to something like a GtkEntry, we need to set the mnemonic_widget. */
+  if (gtk_label_get_use_underline (GTK_LABEL (widget))
+      && !gb_label_find_mnemonic_widget (widget))
     {
-      label_text = glade_util_get_label_text (widget);
-      /* If there is an underlined accelerator, set up the accel signal. */
-      if (strchr (label_text, '_'))
+      gchar *target_name = gtk_object_get_data (GTK_OBJECT (widget),
+						focus_target_p);
+      if (target_name)
+	accel_target = glade_util_find_widget (gtk_widget_get_toplevel (widget), target_name);
+
+      if (!accel_target)
+	accel_target = glade_util_find_default_accelerator_target (widget);
+
+      if (accel_target)
 	{
-	  if (!glade_util_find_parent_button (widget))
-	    {
-	      target_name = gtk_object_get_data (GTK_OBJECT (widget),
-						 AccelTarget);
-	      if (target_name)
-		accel_target = glade_util_find_widget (gtk_widget_get_toplevel (widget), target_name);
-	      signal_name = "grab_focus";
-	    }
-
-	  if (!accel_target)
-	    {
-	      accel_target = glade_util_find_default_accelerator_target (widget, &signal_name);
-	    }
-
-	  if (accel_target)
-	    {
-	      source_add_decl (data, "  guint %s_key;\n", data->real_wname);
-	      source_add (data, "  %s = gtk_label_new (\"\");\n", data->wname);
-	      source_add (data,
-			  "  %s_key = gtk_label_parse_uline (GTK_LABEL (%s),\n"
-			  "                                   %s);\n",
-			  data->real_wname, data->wname,
-			  source_make_string (label_text, data->use_gettext));
-
-	      data->need_accel_group = TRUE;
-	      target_name = gtk_widget_get_name (accel_target);
-	      target_name = source_create_valid_identifier (target_name);
-	      source_add_to_buffer (data, GLADE_ACCELERATORS,
-				    "  gtk_widget_add_accelerator (%s, \"%s\", accel_group,\n"
-				    "                              %s_key, GDK_MOD1_MASK, 0);\n",
-				    target_name, signal_name,
-				    data->real_wname);
-	      g_free (target_name);
-	    }
-	  else
-	    {
-	      source_add (data, "  %s = gtk_label_new (\"\");\n", data->wname);
-	      source_add (data,
-			  "  gtk_label_parse_uline (GTK_LABEL (%s),\n"
-			  "                         %s);\n",
-			  data->wname,
-			  source_make_string (label_text, data->use_gettext));
-	    }
+	  target_name = (gchar*) gtk_widget_get_name (accel_target);
+	  target_name = source_create_valid_identifier (target_name);
+	  source_add_to_buffer (data, GLADE_ACCELERATORS,
+		"  gtk_label_set_mnemonic_widget (GTK_LABEL (%s), %s);\n",
+				data->wname, target_name);
+	  g_free (target_name);
 	}
-      else
-	{
-	  source_add (data, "  %s = gtk_label_new (%s);\n", data->wname,
-		      source_make_string (label_text, data->use_gettext));
-	}
-      g_free (label_text);
     }
 
-  gb_widget_write_standard_source (widget, data);
-
   if (GTK_LABEL (widget)->jtype != GTK_JUSTIFY_CENTER)
     {
       for (i = 0; i < sizeof (GbJustifyValues) / sizeof (GbJustifyValues[0]);
@@ -486,6 +627,11 @@ gb_label_write_source (GtkWidget * widget, GbWidgetWriteSourceData * data)
     source_add (data, "  gtk_label_set_line_wrap (GTK_LABEL (%s), TRUE);\n",
 		data->wname);
 
+  if (gtk_label_get_selectable (GTK_LABEL (widget)))
+    source_add (data, "  gtk_label_set_selectable (GTK_LABEL (%s), TRUE);\n",
+		data->wname);
+
+
   if (fabs (GTK_MISC (widget)->xalign - 0.5) > 0.0001
       || fabs (GTK_MISC (widget)->yalign - 0.5) > 0.0001)
     source_add (data, "  gtk_misc_set_alignment (GTK_MISC (%s), %g, %g);\n",
@@ -497,6 +643,38 @@ gb_label_write_source (GtkWidget * widget, GbWidgetWriteSourceData * data)
 }
 
 
+/*
+ * Writes the source code needed to create this widget.
+ * You have to output everything necessary to create the widget here, though
+ * there are some convenience functions to help.
+ */
+static void
+gb_label_write_source (GtkWidget * widget, GbWidgetWriteSourceData * data)
+{
+  if (data->create_widget)
+    {
+      const gchar *label_text = gtk_label_get_label (GTK_LABEL (widget));
+
+      if (gtk_label_get_use_underline (GTK_LABEL (widget)))
+	source_add (data, "  %s = gtk_label_new_with_mnemonic (%s);\n",
+		    data->wname,
+		    source_make_string (label_text, data->use_gettext));
+      else
+	source_add (data, "  %s = gtk_label_new (%s);\n",
+		    data->wname,
+		    source_make_string (label_text, data->use_gettext));
+    }
+
+  gb_widget_write_standard_source (widget, data);
+
+  gb_label_write_standard_source (widget, data,
+				  Label, UseUnderline, UseMarkup,
+				  Justify, Wrap, Selectable,
+				  XAlign, YAlign, XPad, YPad,
+				  FocusTarget);
+}
+
+
 
 /*
  * Initializes the GbWidget structure.
@@ -507,7 +685,8 @@ GbWidget *
 gb_label_init ()
 {
   /* Initialise the GTK type */
-  gtk_label_get_type ();
+  volatile GtkType type;
+  type = gtk_label_get_type ();
 
   /* Initialize the GbWidget structure */
   gb_widget_init_struct (&gbwidget);
