@@ -27,6 +27,7 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+#include <math.h>
 #include <gtk/gtkprogressbar.h>
 #include "../gb.h"
 
@@ -39,31 +40,22 @@
  */
 static GbWidget gbwidget;
 
-static gchar *Value = "ProgressBar|GtkProgress::value";
-static gchar *MinValue = "ProgressBar|GtkProgress::lower";
-static gchar *MaxValue = "ProgressBar|GtkProgress::upper";
-static gchar *ActivityMode = "ProgressBar|GtkProgress::activity_mode";
-static gchar *Style = "GtkProgressBar::bar_style";
 static gchar *Orientation = "GtkProgressBar::orientation";
+static gchar *Fraction = "GtkProgressBar::fraction";
+static gchar *PulseStep = "GtkProgressBar::pulse_step";
+static gchar *Text = "GtkProgressBar::text";
+
+static gchar *ActivityMode = "ProgressBar|GtkProgress::activity_mode";
 static gchar *ShowText = "ProgressBar|GtkProgress::show_text";
 static gchar *XAlign = "GtkProgressBar::text_xalign";
 static gchar *YAlign = "GtkProgressBar::text_yalign";
-static gchar *Format = "GtkProgressBar::format";
 
-
-static const gchar *GbStyleChoices[] =
-{"Continuous", "Discrete", NULL};
-
-static const gint GbStyleValues[] =
-{
-  GTK_PROGRESS_CONTINUOUS,
-  GTK_PROGRESS_DISCRETE
-};
-static const gchar *GbStyleSymbols[] =
-{
-  "GTK_PROGRESS_CONTINUOUS",
-  "GTK_PROGRESS_DISCRETE"
-};
+#if 0
+static gchar *Value = "ProgressBar|GtkProgress::value";
+static gchar *MinValue = "ProgressBar|GtkProgress::lower";
+static gchar *MaxValue = "ProgressBar|GtkProgress::upper";
+static gchar *Style = "GtkProgressBar::bar_style";
+#endif
 
 
 static const gchar *GbOrientationChoices[] =
@@ -115,26 +107,22 @@ gb_progress_bar_new (GbWidgetNewData * data)
 static void
 gb_progress_bar_create_properties (GtkWidget * widget, GbWidgetCreateArgData * data)
 {
-  property_add_float (Value, _("Value:"), _("The initial value"));
-  property_add_float (MinValue, _("Minimum Value:"), _("The minimum value"));
-  property_add_float (MaxValue, _("Maximum Value:"), _("The maximum value"));
-
-  property_add_choice (Style, _("Style:"),
-		       _("The style of the progress bar"),
-		       GbStyleChoices);
   property_add_choice (Orientation, _("Orientation:"),
 		       _("The orientation of the progress bar's contents"),
 		       GbOrientationChoices);
+  property_add_float_range (Fraction, _("Fraction:"),
+			    _("The fraction of work that has been completed"),
+			    0, 1, 0.01, 0.1, 0.01, 2);
+  property_add_float_range (PulseStep, _("Pulse Step:"),
+			    _("The fraction of work to move the bouncing block when pulsed"),
+			    0, 1, 0.01, 0.1, 0.01, 2);
+  property_add_string (Text, _("Text:"),
+		       _("The text to display over the progress bar"));
 
   property_add_bool (ActivityMode, _("Activity Mode:"), 
                      _("If the progress bar should act like the front of Kit's car"));
   property_add_bool (ShowText, _("Show Text:"), 
                      _("If the text should be shown in the progress bar"));
-  property_add_string (Format, _("Format:"),
-		       _("The format of the text, similar to sprintf. "
-			 "Use %P, %V, %L and %U to insert the percentage, "
-			 "value, lower bound and upper bound respectively. "
-			 "Use %1V or %2V to show 1 or 2 decimal places."));
   property_add_float_range (XAlign, _("X Align:"),
 			    _("The horizontal alignment of the text"),
 			    0, 1, 0.01, 0.1, 0.01, 2);
@@ -154,32 +142,21 @@ gb_progress_bar_create_properties (GtkWidget * widget, GbWidgetCreateArgData * d
 static void
 gb_progress_bar_get_properties(GtkWidget *widget, GbWidgetGetArgData * data)
 {
-  GtkAdjustment *adjustment;
   gint i;
 
-  adjustment = GTK_PROGRESS (widget)->adjustment;
-
-  gb_widget_output_float (data, Value, adjustment->value);
-  gb_widget_output_float (data, MinValue, adjustment->lower);
-  gb_widget_output_float (data, MaxValue, adjustment->upper);
-
-  for (i = 0; i < sizeof (GbStyleValues) / sizeof (GbStyleValues[0]); i++)
-    {
-      if (GbStyleValues[i] == GTK_PROGRESS_BAR (widget)->bar_style)
-	gb_widget_output_choice (data, Style, i, GbStyleSymbols[i]);
-    }
-
   for (i = 0; i < sizeof (GbOrientationValues) / sizeof (GbOrientationValues[0]); i++)
     {
       if (GbOrientationValues[i] == GTK_PROGRESS_BAR (widget)->orientation)
 	gb_widget_output_choice (data, Orientation, i, GbOrientationSymbols[i]);
     }
   
+    gb_widget_output_float (data, Fraction, gtk_progress_bar_get_fraction (GTK_PROGRESS_BAR (widget)));
+    gb_widget_output_float (data, PulseStep, gtk_progress_bar_get_pulse_step (GTK_PROGRESS_BAR (widget)));
+    gb_widget_output_translatable_string (data, Text, gtk_progress_bar_get_text (GTK_PROGRESS_BAR (widget)));
+
     gb_widget_output_bool (data, ActivityMode, GTK_PROGRESS (widget)->activity_mode);
 
     gb_widget_output_bool (data, ShowText, GTK_PROGRESS (widget)->show_text);
-    gb_widget_output_translatable_string (data, Format,
-					  GTK_PROGRESS (widget)->format);
     gb_widget_output_float (data, XAlign, GTK_PROGRESS (widget)->x_align);
     gb_widget_output_float (data, YAlign, GTK_PROGRESS (widget)->y_align);
 }
@@ -195,63 +172,12 @@ gb_progress_bar_get_properties(GtkWidget *widget, GbWidgetGetArgData * data)
 static void
 gb_progress_bar_set_properties(GtkWidget * widget, GbWidgetSetArgData * data)
 {
-  GtkAdjustment *adjustment;
-  gfloat value, min, max, xalign, yalign;
-  gboolean activityMode, showText;
-  gboolean configure_progress = FALSE, set_alignment = FALSE;
-  gchar *style, *orientation, *format;
+  gfloat xalign, yalign, fraction, pulse_step;
+  gboolean activityMode, showText, set_alignment = FALSE;
+  gchar *orientation, *text;
   gint i;
  
-  adjustment = GTK_PROGRESS (widget)->adjustment;
-
-  value = gb_widget_input_float (data, Value);
-  if (data->apply)
-    configure_progress = TRUE;
-  else
-    value = adjustment->value;
-
-  min = gb_widget_input_float (data, MinValue);
-  if (data->apply)
-    configure_progress = TRUE;
-  else
-    min = adjustment->lower;
-
-  max = gb_widget_input_float (data, MaxValue);
-  if (data->apply)
-    configure_progress = TRUE;
-  else
-    min = adjustment->upper;
-
-  if (configure_progress)
-    {
-      /* Make sure we use valid values. */
-      max = MAX (max, min);
-      value = MAX (value, min);
-      value = MIN (value, max);
-      gtk_progress_configure (GTK_PROGRESS (widget), value, min, max);
-    }
-
-
-  style = gb_widget_input_choice (data, Style);
-  if (data->apply)
-    {
-      for (i = 0; i < sizeof (GbStyleValues) / sizeof (GbStyleValues[0]); i++)
-	{
-	  if (!strcmp (style, GbStyleChoices[i])
-	      || !strcmp (style, GbStyleSymbols[i]))
-	    {
-	      gtk_progress_bar_set_bar_style (GTK_PROGRESS_BAR (widget),
-					      GbStyleValues[i]);
-	      /* FIXME: GTK+ 1.2 workaround. If we don't do this it doesn't
-		 recalculate the number of blocks to show. */
-	      gtk_adjustment_value_changed (GTK_PROGRESS (widget)->adjustment);
-	      break;
-	    }
-	}
-    }
-
   orientation = gb_widget_input_choice (data, Orientation);
-
   if (data->apply)
     {
       for (i = 0;
@@ -268,6 +194,29 @@ gb_progress_bar_set_properties(GtkWidget * widget, GbWidgetSetArgData * data)
 	}
     }
 
+  fraction = gb_widget_input_float (data, Fraction);
+  if (data->apply)
+    gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (widget), fraction);
+
+  pulse_step = gb_widget_input_float (data, PulseStep);
+  if (data->apply)
+    gtk_progress_bar_set_pulse_step (GTK_PROGRESS_BAR (widget), pulse_step);
+
+  text = gb_widget_input_string (data, Text);
+  if (data->apply)
+    {
+      if (text && *text)
+	{
+	  gtk_progress_bar_set_text (GTK_PROGRESS_BAR (widget), text);
+	}
+      else
+	{
+	  /* Reset it to the original format string. This avoids calling
+	   gtk_progress_bar_set_text() as that sets use_text_format to FALSE.*/
+	  gtk_progress_set_format_string (GTK_PROGRESS (widget), "%P %%");
+	}
+    }
+
   activityMode = gb_widget_input_bool (data, ActivityMode);
   if (data->apply)
     gtk_progress_set_activity_mode (GTK_PROGRESS (widget), activityMode);
@@ -276,10 +225,6 @@ gb_progress_bar_set_properties(GtkWidget * widget, GbWidgetSetArgData * data)
   if (data->apply)
     gtk_progress_set_show_text (GTK_PROGRESS (widget), showText);
 
-  format = gb_widget_input_string (data, Format);
-  if (data->apply)
-    gtk_progress_set_format_string (GTK_PROGRESS (widget), format);
-
   xalign = gb_widget_input_float (data, XAlign);
   if (data->apply)
     set_alignment = TRUE;
@@ -305,37 +250,16 @@ gb_progress_bar_set_properties(GtkWidget * widget, GbWidgetSetArgData * data)
 static void
 gb_progress_bar_write_source (GtkWidget * widget, GbWidgetWriteSourceData * data)
 {
-  GtkAdjustment *adjustment;
+  gfloat fraction, pulse_step;
+  const char *text;
   gint i;
 
-  adjustment = GTK_PROGRESS (widget)->adjustment;
-
   if (data->create_widget)
     {
       source_add (data, "  %s = gtk_progress_bar_new ();\n", data->wname);
     }
   gb_widget_write_standard_source (widget, data);
 
-  if (adjustment->value != 0 || adjustment->lower != 0
-      || adjustment->upper != 100)
-    {
-      source_add (data,
-		  "  gtk_progress_configure (GTK_PROGRESS (%s), %g, %g, %g);\n", 
-		  data->wname, adjustment->value,
-		  adjustment->lower, adjustment->upper);
-    }
-
-  if (GTK_PROGRESS_BAR (widget)->bar_style != GTK_PROGRESS_CONTINUOUS)
-    {
-      for (i = 0; i < sizeof (GbStyleValues) / sizeof (GbStyleValues[0]); i++)
-        {
-          if (GbStyleValues[i] == GTK_PROGRESS_BAR (widget)->bar_style)
-            source_add (data,
-		    "  gtk_progress_bar_set_bar_style (GTK_PROGRESS_BAR (%s), %s);\n", 
-                        data->wname, GbStyleSymbols[i]);
-        }
-    }
-
   if (GTK_PROGRESS_BAR (widget)->orientation != GTK_PROGRESS_LEFT_TO_RIGHT)
     {
       for (i = 0; i < sizeof (GbOrientationValues) / sizeof (GbOrientationValues[0]); i++)
@@ -347,6 +271,18 @@ gb_progress_bar_write_source (GtkWidget * widget, GbWidgetWriteSourceData * data
         }
     }
 
+  fraction = gtk_progress_bar_get_fraction (GTK_PROGRESS_BAR (widget));
+  if (fraction >= GLADE_EPSILON)
+    source_add (data,
+	"  gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (%s), %g);\n", 
+		data->wname, fraction);
+
+  pulse_step = gtk_progress_bar_get_pulse_step (GTK_PROGRESS_BAR (widget));
+  if (fabs (pulse_step - 0.1) >= GLADE_EPSILON)
+    source_add (data,
+	"  gtk_progress_bar_set_pulse_step (GTK_PROGRESS_BAR (%s), %g);\n", 
+		data->wname, pulse_step);
+
   if (GTK_PROGRESS (widget)->activity_mode)
     {
       source_add( data,
@@ -361,13 +297,14 @@ gb_progress_bar_write_source (GtkWidget * widget, GbWidgetWriteSourceData * data
                   data->wname);
     }
 
-  if (strcmp (GTK_PROGRESS (widget)->format, "%P %%"))
+
+  text = gtk_progress_bar_get_text (GTK_PROGRESS_BAR (widget));
+  if (text && *text)
     {
       source_add( data,
-                  "  gtk_progress_set_format_string (GTK_PROGRESS (%s), %s);\n",
+                  "  gtk_progress_set_text (GTK_PROGRESS (%s), %s);\n",
                   data->wname,
-		  source_make_string (GTK_PROGRESS (widget)->format,
-				      data->use_gettext));
+		  source_make_string (text, data->use_gettext));
     }
 
   if (GTK_PROGRESS (widget)->x_align != 0.5
@@ -391,7 +328,8 @@ GbWidget *
 gb_progress_bar_init ()
 {
   /* Initialise the GTK type */
-  gtk_progress_bar_get_type ();
+  volatile GtkType type;
+  type = gtk_progress_bar_get_type ();
 
   /* Initialize the GbWidget structure */
   gb_widget_init_struct (&gbwidget);
