@@ -793,7 +793,9 @@ gb_window_write_standard_source (GtkWidget * widget,
 		      "      gtk_window_set_icon (GTK_WINDOW (%s), %s);\n"
 		      "      gdk_pixbuf_unref (%s);\n"
 		      "    }\n",
-		      pixbuf_name, pixbuf_name, pixbuf_name, pixbuf_name);
+		      pixbuf_name,
+		      data->wname, pixbuf_name,
+		      pixbuf_name);
 
 	  g_free (pixbuf_name);
 	}
