@@ -18,7 +18,7 @@
  */
 
 /*
- * This file contains general Gnome-related decalarations & code.
+ * This file contains general Gnome-related declarations & code.
  */
 
 #include <config.h>
@@ -54,27 +54,27 @@ static gboolean glade_gnome_is_first_radio_menu_item_in_group (GtkMenuItem *menu
  * so we don't duplicate them and we use simple convenience functions for them.
  */
 
-/* Choices for the GnomeDockItem placement. */
+/* Choices for the BonoboDockItem placement. */
 const gint GladePlacementValues[] =
 {
-  GNOME_DOCK_TOP,
-  GNOME_DOCK_RIGHT,
-  GNOME_DOCK_BOTTOM,
-  GNOME_DOCK_LEFT,
-  GNOME_DOCK_FLOATING
+  BONOBO_DOCK_TOP,
+  BONOBO_DOCK_RIGHT,
+  BONOBO_DOCK_BOTTOM,
+  BONOBO_DOCK_LEFT,
+  BONOBO_DOCK_FLOATING
 };
 const gint GladePlacementSize = sizeof (GladePlacementValues) / sizeof (gint);
 const gchar *GladePlacementSymbols[] =
 {
-  "GNOME_DOCK_TOP",
-  "GNOME_DOCK_RIGHT",
-  "GNOME_DOCK_BOTTOM",
-  "GNOME_DOCK_LEFT",
-  "GNOME_DOCK_FLOATING"
+  "BONOBO_DOCK_TOP",
+  "BONOBO_DOCK_RIGHT",
+  "BONOBO_DOCK_BOTTOM",
+  "BONOBO_DOCK_LEFT",
+  "BONOBO_DOCK_FLOATING"
 };
 
 
-/* Choices for the GnomeDockItem Orientation. */
+/* Choices for the BonoboDockItem Orientation. */
 const gchar *GladeOrientationChoices[] =
 {
   "Horizontal",
@@ -240,636 +240,6 @@ const gchar* GladeStockMenuItemSymbols[] =
 const gint GladeStockMenuItemSize = sizeof (GladeStockMenuItemSymbols) / sizeof (GladeStockMenuItemSymbols[0]);
 
 
-/* Returns the index of the given stock menu pixmap name in the
-   GladeStockMenuPixmapChoices[] and related arrays, or -1 if the icon name
-   isn't found. The icon name should be the gettext-translated name. */
-gint
-glade_gnome_get_stock_menu_pixmap_index (const gchar *icon_name)
-{
-  static GHashTable *stock_menu_icon_hash = NULL;
-
-  gint stock_index, i;
-
-  /* If this is the first call to this function, we create the hash. */
-  if (stock_menu_icon_hash == NULL)
-    {
-      stock_menu_icon_hash = g_hash_table_new (g_str_hash, g_str_equal);
-
-      for (i = 0; GladeStockMenuPixmapChoices[i]; i++)
-	{
-	  /* We add 1 to the index to distinguish an index of 0 from an
-	     unknown icon name. */
-	  g_hash_table_insert (stock_menu_icon_hash,
-			       _(GladeStockMenuPixmapChoices[i]),
-			       GINT_TO_POINTER (i + 1));
-	}
-    }
-
-  /* Now lookup the icon name in the hash. */
-  stock_index = GPOINTER_TO_INT (g_hash_table_lookup (stock_menu_icon_hash,
-						      icon_name));
-
-  /* Remember to subtract 1 to get the real index. This also means that
-     if the icon_name wasn't found we return -1. */
-  return stock_index - 1;
-}
-
-
-/*************************************************************************
- * Stock Gnome Menu Pixmaps.
- *************************************************************************/
-
-/* These were generated from gnome-stock.h with a Perl script, using
-   gnome-libs 1.0.16. */
-const gchar *GladeStockMenuPixmapChoices[] =
-{
-  N_("Blank"),
-  N_("New"),
-  N_("Save"),
-  N_("Save As"),
-  N_("Revert"),
-  N_("Open"),
-  N_("Close"),
-  N_("Quit"),
-  N_("Cut"),
-  N_("Copy"),
-  N_("Paste"),
-  N_("Properties"),
-  N_("Preferences"),
-  N_("About"),
-  N_("Scores"),
-  N_("Undo"),
-  N_("Redo"),
-  N_("Print"),
-  N_("Search"),
-  N_("Search/Replace"),
-  N_("Back"),
-  N_("Forward"),
-  N_("First"),
-  N_("Last"),
-  N_("Home"),
-  N_("Stop"),
-  N_("Refresh"),
-  N_("Mail"),
-  N_("Receive Mail"),
-  N_("Send Mail"),
-  N_("Reply to Mail"),
-  N_("Forward Mail"),
-  N_("New Mail"),
-  N_("Trash"),
-  N_("Trash Full"),
-  N_("Undelete"),
-  N_("Timer"),
-  N_("Timer Stopped"),
-  N_("Spellchecker"),
-  N_("Microphone"),
-  N_("Line In"),
-  N_("Cdrom"),
-  N_("Volume"),
-  N_("Midi"),
-  N_("Book Red"),
-  N_("Book Green"),
-  N_("Book Blue"),
-  N_("Book Yellow"),
-  N_("Book Open"),
-  N_("Convert"),
-  N_("Jump To"),
-  N_("Up"),
-  N_("Down"),
-  N_("Top"),
-  N_("Bottom"),
-  N_("Attach"),
-  N_("Index"),
-  N_("Font"),
-  N_("Exec"),
-  N_("Left"),
-  N_("Right"),
-  N_("Center"),
-  N_("Justify"),
-  N_("Bold"),
-  N_("Italic"),
-  N_("Underline"),
-  N_("Strikeout"),
-
-  NULL
-};
-
-const gchar *GladeStockMenuPixmapValues[] =
-{
-  GNOME_STOCK_MENU_BLANK,
-  GNOME_STOCK_MENU_NEW,
-  GNOME_STOCK_MENU_SAVE,
-  GNOME_STOCK_MENU_SAVE_AS,
-  GNOME_STOCK_MENU_REVERT,
-  GNOME_STOCK_MENU_OPEN,
-  GNOME_STOCK_MENU_CLOSE,
-  GNOME_STOCK_MENU_EXIT,
-  GNOME_STOCK_MENU_CUT,
-  GNOME_STOCK_MENU_COPY,
-  GNOME_STOCK_MENU_PASTE,
-  GNOME_STOCK_MENU_PROP,
-  GNOME_STOCK_MENU_PREF,
-  GNOME_STOCK_MENU_ABOUT,
-  GNOME_STOCK_MENU_SCORES,
-  GNOME_STOCK_MENU_UNDO,
-  GNOME_STOCK_MENU_REDO,
-  GNOME_STOCK_MENU_PRINT,
-  GNOME_STOCK_MENU_SEARCH,
-  GNOME_STOCK_MENU_SRCHRPL,
-  GNOME_STOCK_MENU_BACK,
-  GNOME_STOCK_MENU_FORWARD,
-  GNOME_STOCK_MENU_FIRST,
-  GNOME_STOCK_MENU_LAST,
-  GNOME_STOCK_MENU_HOME,
-  GNOME_STOCK_MENU_STOP,
-  GNOME_STOCK_MENU_REFRESH,
-  GNOME_STOCK_MENU_MAIL,
-  GNOME_STOCK_MENU_MAIL_RCV,
-  GNOME_STOCK_MENU_MAIL_SND,
-  GNOME_STOCK_MENU_MAIL_RPL,
-  GNOME_STOCK_MENU_MAIL_FWD,
-  GNOME_STOCK_MENU_MAIL_NEW,
-  GNOME_STOCK_MENU_TRASH,
-  GNOME_STOCK_MENU_TRASH_FULL,
-  GNOME_STOCK_MENU_UNDELETE,
-  GNOME_STOCK_MENU_TIMER,
-  GNOME_STOCK_MENU_TIMER_STOP,
-  GNOME_STOCK_MENU_SPELLCHECK,
-  GNOME_STOCK_MENU_MIC,
-  GNOME_STOCK_MENU_LINE_IN,
-  GNOME_STOCK_MENU_CDROM,
-  GNOME_STOCK_MENU_VOLUME,
-  GNOME_STOCK_MENU_MIDI,
-  GNOME_STOCK_MENU_BOOK_RED,
-  GNOME_STOCK_MENU_BOOK_GREEN,
-  GNOME_STOCK_MENU_BOOK_BLUE,
-  GNOME_STOCK_MENU_BOOK_YELLOW,
-  GNOME_STOCK_MENU_BOOK_OPEN,
-  GNOME_STOCK_MENU_CONVERT,
-  GNOME_STOCK_MENU_JUMP_TO,
-  GNOME_STOCK_MENU_UP,
-  GNOME_STOCK_MENU_DOWN,
-  GNOME_STOCK_MENU_TOP,
-  GNOME_STOCK_MENU_BOTTOM,
-  GNOME_STOCK_MENU_ATTACH,
-  GNOME_STOCK_MENU_INDEX,
-  GNOME_STOCK_MENU_FONT,
-  GNOME_STOCK_MENU_EXEC,
-  GNOME_STOCK_MENU_ALIGN_LEFT,
-  GNOME_STOCK_MENU_ALIGN_RIGHT,
-  GNOME_STOCK_MENU_ALIGN_CENTER,
-  GNOME_STOCK_MENU_ALIGN_JUSTIFY,
-  GNOME_STOCK_MENU_TEXT_BOLD,
-  GNOME_STOCK_MENU_TEXT_ITALIC,
-  GNOME_STOCK_MENU_TEXT_UNDERLINE,
-  GNOME_STOCK_MENU_TEXT_STRIKEOUT,
-
-  NULL
-};
-
-const gchar *GladeStockMenuPixmapSymbols[] =
-{
-  "GNOME_STOCK_MENU_BLANK",
-  "GNOME_STOCK_MENU_NEW",
-  "GNOME_STOCK_MENU_SAVE",
-  "GNOME_STOCK_MENU_SAVE_AS",
-  "GNOME_STOCK_MENU_REVERT",
-  "GNOME_STOCK_MENU_OPEN",
-  "GNOME_STOCK_MENU_CLOSE",
-  "GNOME_STOCK_MENU_EXIT",
-  "GNOME_STOCK_MENU_CUT",
-  "GNOME_STOCK_MENU_COPY",
-  "GNOME_STOCK_MENU_PASTE",
-  "GNOME_STOCK_MENU_PROP",
-  "GNOME_STOCK_MENU_PREF",
-  "GNOME_STOCK_MENU_ABOUT",
-  "GNOME_STOCK_MENU_SCORES",
-  "GNOME_STOCK_MENU_UNDO",
-  "GNOME_STOCK_MENU_REDO",
-  "GNOME_STOCK_MENU_PRINT",
-  "GNOME_STOCK_MENU_SEARCH",
-  "GNOME_STOCK_MENU_SRCHRPL",
-  "GNOME_STOCK_MENU_BACK",
-  "GNOME_STOCK_MENU_FORWARD",
-  "GNOME_STOCK_MENU_FIRST",
-  "GNOME_STOCK_MENU_LAST",
-  "GNOME_STOCK_MENU_HOME",
-  "GNOME_STOCK_MENU_STOP",
-  "GNOME_STOCK_MENU_REFRESH",
-  "GNOME_STOCK_MENU_MAIL",
-  "GNOME_STOCK_MENU_MAIL_RCV",
-  "GNOME_STOCK_MENU_MAIL_SND",
-  "GNOME_STOCK_MENU_MAIL_RPL",
-  "GNOME_STOCK_MENU_MAIL_FWD",
-  "GNOME_STOCK_MENU_MAIL_NEW",
-  "GNOME_STOCK_MENU_TRASH",
-  "GNOME_STOCK_MENU_TRASH_FULL",
-  "GNOME_STOCK_MENU_UNDELETE",
-  "GNOME_STOCK_MENU_TIMER",
-  "GNOME_STOCK_MENU_TIMER_STOP",
-  "GNOME_STOCK_MENU_SPELLCHECK",
-  "GNOME_STOCK_MENU_MIC",
-  "GNOME_STOCK_MENU_LINE_IN",
-  "GNOME_STOCK_MENU_CDROM",
-  "GNOME_STOCK_MENU_VOLUME",
-  "GNOME_STOCK_MENU_MIDI",
-  "GNOME_STOCK_MENU_BOOK_RED",
-  "GNOME_STOCK_MENU_BOOK_GREEN",
-  "GNOME_STOCK_MENU_BOOK_BLUE",
-  "GNOME_STOCK_MENU_BOOK_YELLOW",
-  "GNOME_STOCK_MENU_BOOK_OPEN",
-  "GNOME_STOCK_MENU_CONVERT",
-  "GNOME_STOCK_MENU_JUMP_TO",
-  "GNOME_STOCK_MENU_UP",
-  "GNOME_STOCK_MENU_DOWN",
-  "GNOME_STOCK_MENU_TOP",
-  "GNOME_STOCK_MENU_BOTTOM",
-  "GNOME_STOCK_MENU_ATTACH",
-  "GNOME_STOCK_MENU_INDEX",
-  "GNOME_STOCK_MENU_FONT",
-  "GNOME_STOCK_MENU_EXEC",
-  "GNOME_STOCK_MENU_ALIGN_LEFT",
-  "GNOME_STOCK_MENU_ALIGN_RIGHT",
-  "GNOME_STOCK_MENU_ALIGN_CENTER",
-  "GNOME_STOCK_MENU_ALIGN_JUSTIFY",
-  "GNOME_STOCK_MENU_TEXT_BOLD",
-  "GNOME_STOCK_MENU_TEXT_ITALIC",
-  "GNOME_STOCK_MENU_TEXT_UNDERLINE",
-  "GNOME_STOCK_MENU_TEXT_STRIKEOUT",
-
-  NULL
-};
-
-const gint GladeStockMenuPixmapSize = sizeof (GladeStockMenuPixmapSymbols) / sizeof (GladeStockMenuPixmapSymbols[0]);
-
-
-/* Returns the index of the given stock pixmap name in the
-   GladeStockPixmapChoices[] and related arrays, or -1 if the icon name
-   isn't found. The icon name should be the gettext-translated name. */
-gint
-glade_gnome_get_stock_pixmap_index (const gchar *icon_name)
-{
-  static GHashTable *stock_pixmap_hash = NULL;
-
-  gint stock_index, i;
-
-  /* If this is the first call to this function, we create the hash. */
-  if (stock_pixmap_hash == NULL)
-    {
-      stock_pixmap_hash = g_hash_table_new (g_str_hash, g_str_equal);
-
-      for (i = 0; GladeStockPixmapChoices[i]; i++)
-	{
-	  /* We add 1 to the index to distinguish an index of 0 from an
-	     unknown icon name. */
-	  g_hash_table_insert (stock_pixmap_hash,
-			       _(GladeStockPixmapChoices[i]),
-			       GINT_TO_POINTER (i + 1));
-	}
-    }
-
-  /* Now lookup the icon name in the hash. */
-  stock_index = GPOINTER_TO_INT (g_hash_table_lookup (stock_pixmap_hash,
-						      icon_name));
-
-  /* Remember to subtract 1 to get the real index. This also means that
-     if the icon_name wasn't found we return -1. */
-  return stock_index - 1;
-}
-
-
-/*************************************************************************
- * Stock Gnome Pixmaps.
- *************************************************************************/
-
-/* These were generated from gnome-stock.h with a Perl script, using
-   gnome-libs 1.0.16.
-   Note that ToolBarIndices in gnome/gnomeapp.c uses indexes into this array
-   so if this is changed make sure they are updated. */
-const gchar *GladeStockPixmapChoices[] =
-{
-  N_("None"),
-  N_("New"),
-  N_("Open"),
-  N_("Close"),
-  N_("Revert"),
-  N_("Save"),
-  N_("Save As"),
-  N_("Cut"),
-  N_("Copy"),
-  N_("Paste"),
-  N_("Clear"),
-  N_("Properties"),
-  N_("Preferences"),
-  N_("Help"),
-  N_("Scores"),
-  N_("Print"),
-  N_("Search"),
-  N_("Search/Replace"),
-  N_("Back"),
-  N_("Forward"),
-  N_("First"),
-  N_("Last"),
-  N_("Home"),
-  N_("Stop"),
-  N_("Refresh"),
-  N_("Undo"),
-  N_("Redo"),
-  N_("Timer"),
-  N_("Timer Stopped"),
-  N_("Mail"),
-  N_("Receive Mail"),
-  N_("Send Mail"),
-  N_("Reply to Mail"),
-  N_("Forward Mail"),
-  N_("New Mail"),
-  N_("Trash"),
-  N_("Trash Full"),
-  N_("Undelete"),
-  N_("Spellchecker"),
-  N_("Microphone"),
-  N_("Line In"),
-  N_("Cdrom"),
-  N_("Volume"),
-  N_("Midi"),
-  N_("Book Red"),
-  N_("Book Green"),
-  N_("Book Blue"),
-  N_("Book Yellow"),
-  N_("Book Open"),
-  N_("About"),
-  N_("Quit"),
-  N_("Multiple"),
-  N_("Not"),
-  N_("Convert"),
-  N_("Jump To"),
-  N_("Up"),
-  N_("Down"),
-  N_("Top"),
-  N_("Bottom"),
-  N_("Attach"),
-  N_("Index"),
-  N_("Font"),
-  N_("Exec"),
-  N_("Left"),
-  N_("Right"),
-  N_("Center"),
-  N_("Justify"),
-  N_("Bold"),
-  N_("Italic"),
-  N_("Underline"),
-  N_("Strikeout"),
-  N_("Text Indent"),
-  N_("Text Unindent"),
-  N_("Color Select"),
-  N_("Add"),
-  N_("Remove"),
-  N_("Table Borders"),
-  N_("Table Fill"),
-  N_("Text Bulleted List"),
-  N_("Text Numbered List"),
-
-  NULL
-};
-
-const gchar *GladeStockPixmapValues[] =
-{
-  NULL,
-  GNOME_STOCK_PIXMAP_NEW,
-  GNOME_STOCK_PIXMAP_OPEN,
-  GNOME_STOCK_PIXMAP_CLOSE,
-  GNOME_STOCK_PIXMAP_REVERT,
-  GNOME_STOCK_PIXMAP_SAVE,
-  GNOME_STOCK_PIXMAP_SAVE_AS,
-  GNOME_STOCK_PIXMAP_CUT,
-  GNOME_STOCK_PIXMAP_COPY,
-  GNOME_STOCK_PIXMAP_PASTE,
-  GNOME_STOCK_PIXMAP_CLEAR,
-  GNOME_STOCK_PIXMAP_PROPERTIES,
-  GNOME_STOCK_PIXMAP_PREFERENCES,
-  GNOME_STOCK_PIXMAP_HELP,
-  GNOME_STOCK_PIXMAP_SCORES,
-  GNOME_STOCK_PIXMAP_PRINT,
-  GNOME_STOCK_PIXMAP_SEARCH,
-  GNOME_STOCK_PIXMAP_SRCHRPL,
-  GNOME_STOCK_PIXMAP_BACK,
-  GNOME_STOCK_PIXMAP_FORWARD,
-  GNOME_STOCK_PIXMAP_FIRST,
-  GNOME_STOCK_PIXMAP_LAST,
-  GNOME_STOCK_PIXMAP_HOME,
-  GNOME_STOCK_PIXMAP_STOP,
-  GNOME_STOCK_PIXMAP_REFRESH,
-  GNOME_STOCK_PIXMAP_UNDO,
-  GNOME_STOCK_PIXMAP_REDO,
-  GNOME_STOCK_PIXMAP_TIMER,
-  GNOME_STOCK_PIXMAP_TIMER_STOP,
-  GNOME_STOCK_PIXMAP_MAIL,
-  GNOME_STOCK_PIXMAP_MAIL_RCV,
-  GNOME_STOCK_PIXMAP_MAIL_SND,
-  GNOME_STOCK_PIXMAP_MAIL_RPL,
-  GNOME_STOCK_PIXMAP_MAIL_FWD,
-  GNOME_STOCK_PIXMAP_MAIL_NEW,
-  GNOME_STOCK_PIXMAP_TRASH,
-  GNOME_STOCK_PIXMAP_TRASH_FULL,
-  GNOME_STOCK_PIXMAP_UNDELETE,
-  GNOME_STOCK_PIXMAP_SPELLCHECK,
-  GNOME_STOCK_PIXMAP_MIC,
-  GNOME_STOCK_PIXMAP_LINE_IN,
-  GNOME_STOCK_PIXMAP_CDROM,
-  GNOME_STOCK_PIXMAP_VOLUME,
-  GNOME_STOCK_PIXMAP_MIDI,
-  GNOME_STOCK_PIXMAP_BOOK_RED,
-  GNOME_STOCK_PIXMAP_BOOK_GREEN,
-  GNOME_STOCK_PIXMAP_BOOK_BLUE,
-  GNOME_STOCK_PIXMAP_BOOK_YELLOW,
-  GNOME_STOCK_PIXMAP_BOOK_OPEN,
-  GNOME_STOCK_PIXMAP_ABOUT,
-  GNOME_STOCK_PIXMAP_EXIT,
-  GNOME_STOCK_PIXMAP_MULTIPLE,
-  GNOME_STOCK_PIXMAP_NOT,
-  GNOME_STOCK_PIXMAP_CONVERT,
-  GNOME_STOCK_PIXMAP_JUMP_TO,
-  GNOME_STOCK_PIXMAP_UP,
-  GNOME_STOCK_PIXMAP_DOWN,
-  GNOME_STOCK_PIXMAP_TOP,
-  GNOME_STOCK_PIXMAP_BOTTOM,
-  GNOME_STOCK_PIXMAP_ATTACH,
-  GNOME_STOCK_PIXMAP_INDEX,
-  GNOME_STOCK_PIXMAP_FONT,
-  GNOME_STOCK_PIXMAP_EXEC,
-  GNOME_STOCK_PIXMAP_ALIGN_LEFT,
-  GNOME_STOCK_PIXMAP_ALIGN_RIGHT,
-  GNOME_STOCK_PIXMAP_ALIGN_CENTER,
-  GNOME_STOCK_PIXMAP_ALIGN_JUSTIFY,
-  GNOME_STOCK_PIXMAP_TEXT_BOLD,
-  GNOME_STOCK_PIXMAP_TEXT_ITALIC,
-  GNOME_STOCK_PIXMAP_TEXT_UNDERLINE,
-  GNOME_STOCK_PIXMAP_TEXT_STRIKEOUT,
-  GNOME_STOCK_PIXMAP_TEXT_INDENT,
-  GNOME_STOCK_PIXMAP_TEXT_UNINDENT,
-  GNOME_STOCK_PIXMAP_COLORSELECTOR,
-  GNOME_STOCK_PIXMAP_ADD,
-  GNOME_STOCK_PIXMAP_REMOVE,
-  GNOME_STOCK_PIXMAP_TABLE_BORDERS,
-  GNOME_STOCK_PIXMAP_TABLE_FILL,
-  GNOME_STOCK_PIXMAP_TEXT_BULLETED_LIST,
-  GNOME_STOCK_PIXMAP_TEXT_NUMBERED_LIST,
-
-  NULL
-};
-
-const gchar *GladeStockPixmapSymbols[] =
-{
-  NULL,
-  "GNOME_STOCK_PIXMAP_NEW",
-  "GNOME_STOCK_PIXMAP_OPEN",
-  "GNOME_STOCK_PIXMAP_CLOSE",
-  "GNOME_STOCK_PIXMAP_REVERT",
-  "GNOME_STOCK_PIXMAP_SAVE",
-  "GNOME_STOCK_PIXMAP_SAVE_AS",
-  "GNOME_STOCK_PIXMAP_CUT",
-  "GNOME_STOCK_PIXMAP_COPY",
-  "GNOME_STOCK_PIXMAP_PASTE",
-  "GNOME_STOCK_PIXMAP_CLEAR",
-  "GNOME_STOCK_PIXMAP_PROPERTIES",
-  "GNOME_STOCK_PIXMAP_PREFERENCES",
-  "GNOME_STOCK_PIXMAP_HELP",
-  "GNOME_STOCK_PIXMAP_SCORES",
-  "GNOME_STOCK_PIXMAP_PRINT",
-  "GNOME_STOCK_PIXMAP_SEARCH",
-  "GNOME_STOCK_PIXMAP_SRCHRPL",
-  "GNOME_STOCK_PIXMAP_BACK",
-  "GNOME_STOCK_PIXMAP_FORWARD",
-  "GNOME_STOCK_PIXMAP_FIRST",
-  "GNOME_STOCK_PIXMAP_LAST",
-  "GNOME_STOCK_PIXMAP_HOME",
-  "GNOME_STOCK_PIXMAP_STOP",
-  "GNOME_STOCK_PIXMAP_REFRESH",
-  "GNOME_STOCK_PIXMAP_UNDO",
-  "GNOME_STOCK_PIXMAP_REDO",
-  "GNOME_STOCK_PIXMAP_TIMER",
-  "GNOME_STOCK_PIXMAP_TIMER_STOP",
-  "GNOME_STOCK_PIXMAP_MAIL",
-  "GNOME_STOCK_PIXMAP_MAIL_RCV",
-  "GNOME_STOCK_PIXMAP_MAIL_SND",
-  "GNOME_STOCK_PIXMAP_MAIL_RPL",
-  "GNOME_STOCK_PIXMAP_MAIL_FWD",
-  "GNOME_STOCK_PIXMAP_MAIL_NEW",
-  "GNOME_STOCK_PIXMAP_TRASH",
-  "GNOME_STOCK_PIXMAP_TRASH_FULL",
-  "GNOME_STOCK_PIXMAP_UNDELETE",
-  "GNOME_STOCK_PIXMAP_SPELLCHECK",
-  "GNOME_STOCK_PIXMAP_MIC",
-  "GNOME_STOCK_PIXMAP_LINE_IN",
-  "GNOME_STOCK_PIXMAP_CDROM",
-  "GNOME_STOCK_PIXMAP_VOLUME",
-  "GNOME_STOCK_PIXMAP_MIDI",
-  "GNOME_STOCK_PIXMAP_BOOK_RED",
-  "GNOME_STOCK_PIXMAP_BOOK_GREEN",
-  "GNOME_STOCK_PIXMAP_BOOK_BLUE",
-  "GNOME_STOCK_PIXMAP_BOOK_YELLOW",
-  "GNOME_STOCK_PIXMAP_BOOK_OPEN",
-  "GNOME_STOCK_PIXMAP_ABOUT",
-  "GNOME_STOCK_PIXMAP_EXIT",
-  "GNOME_STOCK_PIXMAP_MULTIPLE",
-  "GNOME_STOCK_PIXMAP_NOT",
-  "GNOME_STOCK_PIXMAP_CONVERT",
-  "GNOME_STOCK_PIXMAP_JUMP_TO",
-  "GNOME_STOCK_PIXMAP_UP",
-  "GNOME_STOCK_PIXMAP_DOWN",
-  "GNOME_STOCK_PIXMAP_TOP",
-  "GNOME_STOCK_PIXMAP_BOTTOM",
-  "GNOME_STOCK_PIXMAP_ATTACH",
-  "GNOME_STOCK_PIXMAP_INDEX",
-  "GNOME_STOCK_PIXMAP_FONT",
-  "GNOME_STOCK_PIXMAP_EXEC",
-  "GNOME_STOCK_PIXMAP_ALIGN_LEFT",
-  "GNOME_STOCK_PIXMAP_ALIGN_RIGHT",
-  "GNOME_STOCK_PIXMAP_ALIGN_CENTER",
-  "GNOME_STOCK_PIXMAP_ALIGN_JUSTIFY",
-  "GNOME_STOCK_PIXMAP_TEXT_BOLD",
-  "GNOME_STOCK_PIXMAP_TEXT_ITALIC",
-  "GNOME_STOCK_PIXMAP_TEXT_UNDERLINE",
-  "GNOME_STOCK_PIXMAP_TEXT_STRIKEOUT",
-  "GNOME_STOCK_PIXMAP_TEXT_INDENT",
-  "GNOME_STOCK_PIXMAP_TEXT_UNINDENT",
-  "GNOME_STOCK_PIXMAP_COLORSELECTOR",
-  "GNOME_STOCK_PIXMAP_ADD",
-  "GNOME_STOCK_PIXMAP_REMOVE",
-  "GNOME_STOCK_PIXMAP_TABLE_BORDERS",
-  "GNOME_STOCK_PIXMAP_TABLE_FILL",
-  "GNOME_STOCK_PIXMAP_TEXT_BULLETED_LIST",
-  "GNOME_STOCK_PIXMAP_TEXT_NUMBERED_LIST",
-
-  NULL
-};
-
-const gint GladeStockPixmapSize = sizeof (GladeStockPixmapSymbols) / sizeof (GladeStockPixmapSymbols[0]);
-
-/*************************************************************************
- * Stock Gnome Buttons.
- *************************************************************************/
-
-/* These must be the index of the corresponding item in the arrays below.
-   They are used when creating GnomeDialog & GnomeMessageBox widgets. */
-const gint GladeStockButtonOK = 1;
-const gint GladeStockButtonApply = 6;
-const gint GladeStockButtonCancel = 2;
-
-const gchar *GladeStockButtonChoices[] =
-{
-  "None",
-  "OK",
-  "Cancel",
-  "Yes",
-  "No",
-  "Close",
-  "Apply",
-  "Help",
-  "Next",
-  "Prev",
-  "Up",
-  "Down",
-  "Font",
-  NULL
-};
-/* Note these are strings rather than gints as we normally use for choices. */
-const gchar* GladeStockButtonValues[] =
-{
-  NULL,
-  GNOME_STOCK_BUTTON_OK,
-  GNOME_STOCK_BUTTON_CANCEL,
-  GNOME_STOCK_BUTTON_YES,
-  GNOME_STOCK_BUTTON_NO,
-  GNOME_STOCK_BUTTON_CLOSE,
-  GNOME_STOCK_BUTTON_APPLY,
-  GNOME_STOCK_BUTTON_HELP,
-  GNOME_STOCK_BUTTON_NEXT,
-  GNOME_STOCK_BUTTON_PREV,
-  GNOME_STOCK_BUTTON_UP,
-  GNOME_STOCK_BUTTON_DOWN,
-  GNOME_STOCK_BUTTON_FONT
-};
-const gchar *GladeStockButtonSymbols[] =
-{
-  NULL,
-  "GNOME_STOCK_BUTTON_OK",
-  "GNOME_STOCK_BUTTON_CANCEL",
-  "GNOME_STOCK_BUTTON_YES",
-  "GNOME_STOCK_BUTTON_NO",
-  "GNOME_STOCK_BUTTON_CLOSE",
-  "GNOME_STOCK_BUTTON_APPLY",
-  "GNOME_STOCK_BUTTON_HELP",
-  "GNOME_STOCK_BUTTON_NEXT",
-  "GNOME_STOCK_BUTTON_PREV",
-  "GNOME_STOCK_BUTTON_UP",
-  "GNOME_STOCK_BUTTON_DOWN",
-  "GNOME_STOCK_BUTTON_FONT"
-};
-
-const gint GladeStockButtonSize = sizeof (GladeStockButtonSymbols) / sizeof (GladeStockButtonSymbols[0]);
-
-
 /*************************************************************************
  * Functions to output GnomeUIInfo structs in the source code.
  *************************************************************************/
@@ -963,22 +333,27 @@ glade_gnome_write_menu_item_source (GtkMenuItem * widget,
   GList *tmp_list;
   GladeSignal *activate_signal = NULL;
   GladeAccelerator *activate_accel = NULL;
-  gchar *handler = NULL;
+  gchar *handler = NULL, *child_uiinfo_name = NULL;
 
   source_buffer = gtk_object_get_data (GTK_OBJECT (GTK_WIDGET (widget)->parent), "glade-uiinfo");
   g_return_if_fail (source_buffer != NULL);
 
-  if (widget->submenu)
-    has_child_menu = TRUE;
-
   /* First check if it is a separator, since they are simple. */
-  if (GTK_BIN (widget)->child == NULL)
+  if (GTK_IS_SEPARATOR_MENU_ITEM (widget) || GTK_BIN (widget)->child == NULL)
     {
       g_string_append (source_buffer, "  GNOMEUIINFO_SEPARATOR,\n");
       glade_gnome_write_menu_item_standard_source (widget, data);
       return;
     }
 
+  if (widget->submenu)
+    {
+      const char *child_menu_name = gtk_widget_get_name (widget->submenu);
+
+      has_child_menu = TRUE;
+      child_uiinfo_name = source_create_valid_identifier (child_menu_name);
+    }
+
 
   /* Find 'activate' handler in widget data. */
   tmp_list = data->widget_data->signals;
@@ -1024,8 +399,8 @@ glade_gnome_write_menu_item_source (GtkMenuItem * widget,
 	  if (has_child_menu)
 	    {
 	      g_string_sprintfa (source_buffer,
-				 "  GNOMEUIINFO_MENU_NEW_SUBTREE (%s_menu_uiinfo),\n",
-				 data->real_wname);
+				 "  GNOMEUIINFO_MENU_NEW_SUBTREE (%s_uiinfo),\n",
+				 child_uiinfo_name);
 	    }
 	  else
 	    {
@@ -1046,16 +421,21 @@ glade_gnome_write_menu_item_source (GtkMenuItem * widget,
       /* See if it is a subtree item. */
       else if (uiinfo->type == GNOME_APP_UI_SUBTREE_STOCK)
 	{
-	  g_string_sprintfa (source_buffer, "  %s (%s_menu_uiinfo),\n",
-			     GladeStockMenuItemSymbols[stock_item_index],
-			     data->real_wname);
-
-	  /* The stock subtree item should really have children, but if it
-	     doesn't, we just output an empty GnomeUIInfo struct here. */
-	  if (!has_child_menu)
+	  if (has_child_menu)
 	    {
+	      g_string_sprintfa (source_buffer, "  %s (%s_uiinfo),\n",
+				 GladeStockMenuItemSymbols[stock_item_index],
+				 child_uiinfo_name);
+	    }
+	  else
+	    {
+	      /* The stock subtree item should really have children, but if it
+		 doesn't, we just output an empty GnomeUIInfo struct here. */
+	      g_string_sprintfa (source_buffer, "  %s (%s_uiinfo),\n",
+				 GladeStockMenuItemSymbols[stock_item_index],
+				 data->real_wname);
 	      g_string_sprintfa (data->source_buffers[GLADE_UIINFO],
-				 "static GnomeUIInfo %s_menu_uiinfo[] =\n{\n"
+				 "static GnomeUIInfo %s_uiinfo[] =\n{\n"
 				 "  GNOMEUIINFO_END\n};\n\n",
 				 data->real_wname);
 	    }
@@ -1073,7 +453,7 @@ glade_gnome_write_menu_item_source (GtkMenuItem * widget,
        tooltip, accelerator, and icon (stock or ordinary), and also check
        if it has a child menu. */
     {
-      gchar *type, *label, *tooltip, *icon_name, *accel_mods;
+      gchar *type, *label, *tooltip, *icon_name = NULL, *accel_mods;
       gchar *pixmap_type, *pixmap_info, *pixmap_filename;
       gchar accel_key_buffer[128];
       gboolean free_pixmap_info;
@@ -1084,17 +464,20 @@ glade_gnome_write_menu_item_source (GtkMenuItem * widget,
       /* Determine if the item has an icon and if it is a stock icon or an
 	 xpm file. */
       free_pixmap_info = FALSE;
-      icon_name = gtk_object_get_data (GTK_OBJECT (widget),
-				       GladeMenuItemIconKey);
+      if (GTK_IS_IMAGE_MENU_ITEM (widget))
+	{
+	  GtkWidget *image;
+	  image = gtk_image_menu_item_get_image (GTK_IMAGE_MENU_ITEM (widget));
+	  if (image)
+	    icon_name = gtk_object_get_data (GTK_OBJECT (image), GladeIconKey);
+	}
       if (icon_name && icon_name[0])
 	{
-	  gint stock_icon_index;
-
-	  stock_icon_index = glade_gnome_get_stock_menu_pixmap_index (icon_name);
-	  if (stock_icon_index >= 0)
+	  if (glade_util_check_is_stock_id (icon_name))
 	    {
 	      pixmap_type = "GNOME_APP_PIXMAP_STOCK";
-	      pixmap_info = (gchar*)GladeStockMenuPixmapSymbols[stock_icon_index];
+	      pixmap_info = g_strdup (source_make_string (icon_name, FALSE));
+	      free_pixmap_info = TRUE;
 	    }
 	  else
 	    {
@@ -1137,7 +520,7 @@ glade_gnome_write_menu_item_source (GtkMenuItem * widget,
 			     "  {\n"
 			     "    %s, %s,\n"
 			     "    %s,\n"
-			     "    %s_menu_uiinfo, NULL, NULL,\n"
+			     "    %s_uiinfo, NULL, NULL,\n"
 			     "    %s, %s,\n"
 			     "    %s, (GdkModifierType) %s, NULL\n"
 			     "  },\n",
@@ -1145,7 +528,7 @@ glade_gnome_write_menu_item_source (GtkMenuItem * widget,
 			     source_make_static_string (label,
 							data->use_gettext),
 			     tooltip ? tooltip : "NULL",
-			     data->real_wname,
+			     child_uiinfo_name,
 			     pixmap_type, pixmap_info,
 			     accel_key_buffer, accel_mods);
 	  
@@ -1216,6 +599,7 @@ glade_gnome_write_menu_item_source (GtkMenuItem * widget,
     }
 
   g_free (handler);
+  g_free (child_uiinfo_name);
 }
 
 
@@ -1253,15 +637,11 @@ glade_gnome_write_menu_item_standard_source (GtkMenuItem * widget,
 
   if (!data->use_component_struct)
     {
-      source_add (data,
-		  "  gtk_widget_ref (%s_uiinfo[%i].widget);\n"
-		  "  gtk_object_set_data_full (GTK_OBJECT (%s), %s,\n"
-		  "                            %s_uiinfo[%i].widget,\n"
-		  "                            (GtkDestroyNotify) gtk_widget_unref);\n",
-		  uiinfo_name, uiinfo_index,
-		  data->component_name,
-		  source_make_string (data->real_wname, FALSE),
-		  uiinfo_name, uiinfo_index);
+      source_add_to_buffer (data, GLADE_OBJECT_HOOKUP,
+			    "  GLADE_HOOKUP_OBJECT (%s, %s_uiinfo[%i].widget, %s);\n",
+			    data->component_name,
+			    uiinfo_name, uiinfo_index,
+			    source_make_string (data->real_wname, FALSE));
     }
 
   if (GTK_IS_CHECK_MENU_ITEM (widget) && wdata->flags & GLADE_ACTIVE)
@@ -1390,13 +770,14 @@ glade_gnome_finish_radio_menu_item_group_source (GtkMenuShell * menu_shell,
 
 
 /* This returns TRUE if the given radio menu item is the first item in a
-   radio group. */
+   radio group. FIXME: radio items must be next to each other in GNOME menus,
+   but we don't enforce that in the menu editor. */
 static gboolean
 glade_gnome_is_first_radio_menu_item_in_group (GtkMenuItem *menuitem)
 {
   GList *children, *elem;
   GtkWidget *prev_item;
-  gchar *group, *prev_group;
+  GSList *group, *prev_group;
 
   children = GTK_MENU_SHELL (GTK_WIDGET (menuitem)->parent)->children;
 
@@ -1414,28 +795,13 @@ glade_gnome_is_first_radio_menu_item_in_group (GtkMenuItem *menuitem)
 
   /* If the item's group isn't the same as the previous item's, it must start
      a new group. */
-  group = gtk_object_get_data (GTK_OBJECT (menuitem),
-			       "GtkRadioMenuItem::group");
-  if (group && group[0] == '\0')
-    group = NULL;
+  group = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (menuitem));
+  prev_group = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (prev_item));
 
-  prev_group = gtk_object_get_data (GTK_OBJECT (prev_item),
-				    "GtkRadioMenuItem::group");
-  if (prev_group && prev_group[0] == '\0')
-    prev_group = NULL;
-
-  if (group)
-    {
-      if (!prev_group || strcmp (group, prev_group))
-	return TRUE;
-    }
+  if (group != prev_group)
+    return TRUE;
   else
-    {
-      if (prev_group)
-	return TRUE;
-    }
-
-  return FALSE;
+    return FALSE;
 }
 
 
@@ -1443,21 +809,6 @@ glade_gnome_is_first_radio_menu_item_in_group (GtkMenuItem *menuitem)
  * Utility Functions.
  *************************************************************************/
 
-gboolean
-glade_gnome_is_gnome_dialog_button (GtkWidget *widget)
-{
-  gchar *child_name;
-
-  if (widget->parent)
-    {
-      child_name = gb_widget_get_child_name (widget->parent);
-      if (child_name && !strcmp (child_name, "GnomeDialog:action_area"))
-	return TRUE;
-    }
-  return FALSE;
-}
-
-
 /* If the given widget is a dock item inside a GnomeApp it returns the
    GnomeApp, otherwise it returns NULL. */
 GnomeApp*
@@ -1465,24 +816,24 @@ glade_gnome_is_app_dock_item (GtkWidget *widget)
 {
   GtkWidget *dock, *app;
 
-  if (!GNOME_IS_DOCK_ITEM (widget))
+  if (!BONOBO_IS_DOCK_ITEM (widget))
     return NULL;
 
   if (!widget->parent)
     return NULL;
 
-  /* Floating items are direct children of a GnomeDock.
-     Other items are children of a GnomeDockBand. */
-  if (GNOME_IS_DOCK (widget->parent))
+  /* Floating items are direct children of a BonoboDock.
+     Other items are children of a BonoboDockBand. */
+  if (BONOBO_IS_DOCK (widget->parent))
     {
       dock = widget->parent;
     }
   else
     {
-      if (!GNOME_IS_DOCK_BAND (widget->parent))
+      if (!BONOBO_IS_DOCK_BAND (widget->parent))
 	return NULL;
       dock = widget->parent->parent;
-      if (!dock || !GNOME_IS_DOCK (dock))
+      if (!dock || !BONOBO_IS_DOCK (dock))
 	return NULL;
     }
 
@@ -1507,7 +858,7 @@ glade_gnome_gettext		     (const gchar	      *text)
 
   s = gettext (text);
   if (s == text)
-    s = dgettext ("gnome-libs", text);
+    s = dgettext (GLADE_LIBGNOMEUI_GETTEXT_PACKAGE, text);
 
   return s;
 #else
@@ -1516,4 +867,166 @@ glade_gnome_gettext		     (const gchar	      *text)
 }
 
 
+/* The indices come from GladeStockMenuItemValues in glade_gnome.c */
+static gint FileMenuIndices[] = { 3, 4, 5, 6, -1, 11 };
+static GnomeUIInfo FileMenu[] =
+{
+  GNOMEUIINFO_MENU_NEW_ITEM (N_("_New"), N_("Create a new file"),
+			     NULL, NULL),
+  GNOMEUIINFO_MENU_OPEN_ITEM (NULL, NULL),
+  GNOMEUIINFO_MENU_SAVE_ITEM (NULL, NULL),
+  GNOMEUIINFO_MENU_SAVE_AS_ITEM (NULL, NULL),
+  GNOMEUIINFO_SEPARATOR,
+  GNOMEUIINFO_MENU_EXIT_ITEM (NULL, NULL),
+  GNOMEUIINFO_END
+};
+
+static gint EditMenuIndices[] = { 14, 15, 16, 18, -1, 24, -1, 29 };
+static GnomeUIInfo EditMenu[] =
+{
+  GNOMEUIINFO_MENU_CUT_ITEM (NULL, NULL),
+  GNOMEUIINFO_MENU_COPY_ITEM (NULL, NULL),
+  GNOMEUIINFO_MENU_PASTE_ITEM (NULL, NULL),
+  GNOMEUIINFO_MENU_CLEAR_ITEM (NULL, NULL),
+  GNOMEUIINFO_SEPARATOR,
+  GNOMEUIINFO_MENU_PROPERTIES_ITEM (NULL, NULL),
+  GNOMEUIINFO_SEPARATOR,
+  GNOMEUIINFO_MENU_PREFERENCES_ITEM (NULL, NULL),
+  GNOMEUIINFO_END
+};
+
+static gint ViewMenuIndices[] = { -1 };
+static GnomeUIInfo ViewMenu[] =
+{
+  GNOMEUIINFO_END
+};
+
+static gint HelpMenuIndices[] = { 38 };
+static GnomeUIInfo HelpMenu[] =
+{
+  GNOMEUIINFO_MENU_ABOUT_ITEM (NULL, NULL),
+  GNOMEUIINFO_END
+};
+
+static gint MainMenuIndices[] = { 2, 13, 26, 37 };
+static GnomeUIInfo MainMenu[] =
+{
+  GNOMEUIINFO_MENU_FILE_TREE (FileMenu),
+  GNOMEUIINFO_MENU_EDIT_TREE (EditMenu),
+  GNOMEUIINFO_MENU_VIEW_TREE (ViewMenu),
+  GNOMEUIINFO_MENU_HELP_TREE (HelpMenu),
+  GNOMEUIINFO_END
+};
+
+
+static void
+glade_gnome_setup_menu_items (GnomeUIInfo *uiinfo, gint *indices)
+{
+  gint i = 0;
+  gchar *name, *dest;
+  const gchar *src;
+
+  while (uiinfo->type != GNOME_APP_UI_ENDOFINFO)
+    {
+      if (uiinfo->widget)
+	{
+	  if (indices[i] == -1)
+	    {
+	      gb_widget_create_from (uiinfo->widget, "separator");
+	    }
+	  else
+	    {
+	      name = g_malloc (strlen (uiinfo->label));
+	      /* Convert spaces to underscores, and ignore periods (e.g. in
+		 "Open...") and underscores (e.g. in "_Open"). */
+	      for (src = uiinfo->label, dest = name; *src; src++)
+		{
+		  if (*src == ' ')
+		    *dest++ = '_';
+		  else if (*src == '.')
+		    continue;
+		  else if (*src == '_')
+		    continue;
+		  else
+		    *dest++ = *src;
+		}
+	      *dest = '\0';
+
+	      gb_widget_create_from (uiinfo->widget, name);
+
+	      gtk_object_set_data (GTK_OBJECT (uiinfo->widget),
+				   GladeMenuItemStockIndexKey,
+				   GINT_TO_POINTER (indices[i]));
+
+	      /* If the item has a child menu, turn it into a GbWidget.
+		 If not, add a default handler to the item. */
+	      if (GTK_IS_MENU_ITEM (uiinfo->widget)
+		  && GTK_MENU_ITEM (uiinfo->widget)->submenu)
+		{
+		  gchar *submenu_name;
+
+		  submenu_name = g_strdup_printf ("%s_menu",
+						  uiinfo->widget->name);
+		  gtk_widget_set_name (GTK_MENU_ITEM (uiinfo->widget)->submenu,
+				       submenu_name);
+		  g_free (submenu_name);
+		  gb_widget_create_from (GTK_MENU_ITEM (uiinfo->widget)->submenu, NULL);
+		}
+	      else
+		{
+		  GladeWidgetData *wdata;
+		  GladeSignal *signal;
+		  gchar *start = "on_", *end = "_activate";
+
+		  signal = g_new (GladeSignal, 1);
+		  signal->name = g_strdup ("activate");
+		  /* Generate a default handler name. */
+		  signal->handler = g_malloc (strlen (uiinfo->widget->name) + 1
+					      + strlen (start) + strlen (end));
+		  strcpy (signal->handler, start);
+		  strcat (signal->handler, uiinfo->widget->name);
+		  strcat (signal->handler, end);
+
+		  signal->object = NULL;
+		  signal->after = FALSE;
+		  signal->data = NULL;
+		  signal->last_modification_time = time (NULL);
+
+		  wdata = gtk_object_get_data (GTK_OBJECT (uiinfo->widget),
+					       GB_WIDGET_DATA_KEY);
+		  wdata->signals = g_list_append (wdata->signals, signal);
+		}
+	    }
+	}
+      uiinfo++;
+      i++;
+    }
+}
+
+
+/* This sets up defaults menus for GtkMenuBar and GnomeApp widgets. */
+void
+glade_gnome_setup_initial_menus (GtkWidget *widget)
+{
+  /* We create a standard menubar and toolbar which the user can edit or
+     simply delete anything they don't want. */
+  if (GNOME_IS_APP (widget))
+    gnome_app_create_menus (GNOME_APP (widget), MainMenu);
+  else if (GTK_IS_MENU_BAR (widget))
+    gnome_app_fill_menu (GTK_MENU_SHELL (widget), MainMenu, NULL, TRUE, 0);
+  else
+    g_assert_not_reached ();
+
+  /* Turn all the menus & menuitems into GbWidgets and add default
+     handlers. */
+  glade_gnome_setup_menu_items (FileMenu, FileMenuIndices);
+  glade_gnome_setup_menu_items (EditMenu, EditMenuIndices);
+  glade_gnome_setup_menu_items (ViewMenu, ViewMenuIndices);
+  glade_gnome_setup_menu_items (HelpMenu, HelpMenuIndices);
+  glade_gnome_setup_menu_items (MainMenu, MainMenuIndices);
+}
+
+
+
+
 #endif /* USE_GNOME */
