@@ -65,10 +65,10 @@ static guint glade_palette_signals[LAST_SIGNAL] = {0};
 static GtkWindowClass *parent_class = NULL;
 
 
-guint
+GType
 glade_palette_get_type (void)
 {
-  static guint palette_type = 0;
+  static GType palette_type = 0;
 
   if (!palette_type)
     {
