@@ -16,12 +16,12 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#include "gladeconfig.h"
-
 #include <string.h>
 
 #include <gtk/gtkmenu.h>
 
+#include "gladeconfig.h"
+
 #include "glade.h"
 #include "gbwidget.h"
 #include "glade_gtk12lib.h"
@@ -43,7 +43,7 @@ static void
 palette_create ()
 {
   glade_palette = glade_palette_new ();
-  gtk_widget_set_uposition (glade_palette, 0, 250);
+  /*gtk_widget_set_uposition (glade_palette, 0, 250);*/
   gtk_window_set_wmclass (GTK_WINDOW (glade_palette), "palette", "Glade");
 
   gtk_signal_connect (GTK_OBJECT (glade_palette), "delete_event",
@@ -115,9 +115,20 @@ on_palette_select_item (GladePalette *palette,
   if (class_name == NULL)
     return;
 
+  /* If we don't have a project open, just reset the selection. Hopefully
+     we'll make the palette insensitive in that case so this won't happen. */
+  if (current_project == NULL)
+    {
+      glade_palette_reset_selection (GLADE_PALETTE (glade_palette), FALSE);
+      return;
+    }
+
   /* See if a toplevel item was selected - a window, dialog or menu.
      If it was, we create a new component and reset the palette. */
-  type = gtk_type_from_name (class_name);
+  type = g_type_from_name (class_name);
+#if 0
+  g_print ("Class Name: %s Type: %li\n", class_name, type);
+#endif
 
   /* We just return if the type isn't found (e.g. for the custom widget). */
   if (type == 0)
